function fout = channelwidth(x,n_diff,parameters)
%fout = channelwidth(x,n_diff,parameters)
%   scaled basal melt rate under ice shelf as a function of position x
%   input arguments:
%       x:  position
%       n_diff: differentiation level, must admit choices n_diff = 0,1,2
%           with n_diff = 0 signifying the accumulation rate itself, n_diff
%           its first derivative etc.
%       parameters: the parameters.model structure passed to
%           outlet_evolve etc. channelwidth has two options, a Gaussian
%           channel constriction and a general polynomial shape. The
%           Gaussian is the default; either choice can be triggered through
%           a field parameters.channel_type with choices 'Gaussian' or
%           'polynomial'. The Gaussian shape is of the form
%               fout = 1 -
%               channel_amplitude*exp(-(x-channel_Gauss_centre)^2/channel_G
%               auss_width)
%           where channel_amplitude, channel_Gauss_centre and
%           channel_Gauss_width can be fields in the parameters structure
%           (default values are .2,1,1)
%           For the polynomial bed, the default setting is fout = 1; if
%           bed_coefficients and a row vector bed_exponents are prescribed
%           as fields in the parameters structure, the bed shape is given by
%               fout = sum(bed_coefficients*x^bed_exponents)
%   output argument:
%       fout: the scaled channel width


if ~isfield(parameters,'channel_type')
    channel_type = 'Gaussian';
else
    channel_type = parameters.channel_type;
end

switch channel_type
    case 'Gaussian'
        if ~isfield(parameters,'channel_amplitude') || isempty(parameters.channel_amplitude)
            amp = .2;
        else
            amp = parameters.channel_amplitude;
        end
        if ~isfield(parameters,'channel_Gauss_width') || isempty(parameters.channel_Gauss_width)
            width = 1;
        else
            width = parameters.channel_Gauss_width;
        end
        if ~isfield(parameters,'channel_Gauss_centre') || isempty(parameters.channel_Gauss_centre)
            centre = 1;
        else
            centre = parameters.channel_Gauss_centre;
        end
        switch n_diff
            case 0
                fout = 1 - amp*exp(-(x-centre)^2/width^2);
            case 1
                fout = 2*(x-centre)/width^2*amp*exp(-(x-centre)^2/width^2);
            case 2
                fout = 2/width^2*amp*exp(-(x-centre)^2/width^2)-4*(x-centre)^2/width^4*amp*exp(-(x-centre)^2/width^2);
            case 3
                fout = -12*(x-centre)/width^4*amp*exp(-(x-centre)^2/width^2)+8*(x-centre)^3/width^6*amp*exp(-(x-centre)^2/width^2);
        end
    case 'polynomial'
        if ~isfield(parameters,'channel_coefficients') || isempty(parameters.channel_coefficients)
            channel_coefficients = 1;
        else
            channel_coefficients = parameters.channel_coefficients;
        end
        if ~isfield(parameters,'channel_exponents') || isempty(parameters.channel_exponents)
            channel_exponents = 0;
        else
            channel_exponents = parameters.channel_exponents;
        end
        X = repmat(x,1,length(channel_coefficients));
        coeffs = repmat(channel_coefficients,length(x),1);
        exponents = repmat(channel_exponents,length(x),1);
        switch n_diff
            case 0
                fout = sum(coeffs.*X.^exponents,2);
            case 1
                exponents_mod = exponents-1;
                exponents_mod(exponents==0) = 1;
                fout = sum(coeffs.*exponents.*X.^exponents_mod,2);
            case 2
                exponents_mod = exponents-2;
                exponents_mod(exponents==0|exponents==1) = 1;
                fout = sum(coeffs.*exponents.*(exponents-1).*X.^exponents_mod,2);
            case 3
                exponents_mod = exponents-3;
                exponents_mod(exponents==0|exponents==1|exponents==2) = 1;
                fout = sum(coeffs.*exponents.*(exponents-1).*(exponents-2).*X.^exponents_mod,2);
        end
end

