function fout = evolve(t,v,n_diff,parameters)
%%fout = evolve(t,v,n,parameters)
%   Right-hand side of the dynamical system representing the boundary layer
%   Input arguments
%       t:  time (redundant, used for code compatibility with
%           shoot_parameters only)
%       v:  (xi,Psi,Q) state variable
%       n_diff: Order of differentiation (compatible with shoot_parameters)
%               n_diff = 0 causes 'evolution' to output the evolution function
%               itself
%               n_diff = 1 causes 'evolve' to output the Jacobian of the
%               evolution function with respect to v
%               n_diff = 2 causes 'evolve' to output the second derivative
%               array of the evolution function with respect to v
%               n_diff = 10 causes 'evolve' to output the gradient of the
%               evolution function with respect to the parameters specified in
%               parameters.type. parameters.type is a logical indexing
%               vector from [C,M,Lambda,H_g] into a parameter vector v_par.
%               For instance, if parameters.type = [3; 2], the v_par =
%               [Lambda; M], and n_diff = 10 causes 'event' to output the
%               gradient of the event function with respect to [Lambda; M].
%               n_diff = 11 causes 'evolve' to output the mixed second
%               derivative array of the evolution function with respect to
%               v and the parameters specified in parameters.type.
%       parameters: Parameter structure containing the physical parameters
%           m,n,r,C,M,Lambda,H_g,theta as fields, plus the additional field
%               type:   Specifies which parameters are being varied (see n_diff
%                       above)
%           Note that 'parameters' here is 'parameters.model' in boundary_layer_shoot 


xi = v(1);
Psi = v(2);
Q = v(3);

n = parameters.n;
m = parameters.m;
r = parameters.r;
C = parameters.C;
M = parameters.M;
theta = parameters.theta;

switch n_diff
    case 0
        fout = zeros(3,1);
        fout(1) = (n+1)^2/n^2*xi^2*Q^(n+1);
        fout(2) = Psi^(-n)/4 - Q^(n+1)*(1-(1-theta)*r)/4 - (2*n+1-n^2)/n^2*Q^(n+1)*xi*Psi ...
            + C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n)/4 ...
            + 2*M*xi^(1/(n+1))*Psi^(1-n)*(1-theta) - (1-r)*M*xi^(-n/(n+1))*Psi^(-n)*(1-theta)/4;
        fout(3) = -xi^(1/(n+1))*Psi^(-n)*Q*M*(1-theta);
        
    case 1
        fout = zeros(3,3);
        
        fout(1,1) = 2*xi*(n+1)^2/n^2*Q^(n+1);
        fout(1,3) = (n+1)^3/n^2*xi^2*Q^n;
        
        fout(2,1) = - (2*n+1-n^2)/n^2*Q^(n+1)*Psi ...
            + ((n^2*(m+1)-n)/(n+1)^2)*C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2-1)*Psi^(-n)/4 ...
            + 1/(n+1)*2*M*xi^(1/(n+1)-1)*Psi^(1-n)*(1-theta) ...
            + n/(n+1)*(1-r)*M*xi^(-n/(n+1)-1)*Psi^(-n)*(1-theta)/4;
        fout(2,2) = - n*Psi^(-n-1)/4 - (2*n+1-n^2)/n^2*Q^(n+1)*xi ...
            - n*C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n-1)/4 ...
            + (1-n)*2*M*xi^(1/(n+1))*Psi^(-n)*(1-theta) ...
            + n*(1-r)*M*xi^(-n/(n+1))*Psi^(-n-1)*(1-theta)/4;
        fout(2,3) = - (n+1)*Q^n*(1-(1-theta)*r)/4 - (n+1)*(2*n+1-n^2)/n^2*Q^n*xi*Psi ...
            - C*theta*Q^(-2)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n)/4;
        
        fout(3,1) = -1/(n+1)*xi^(1/(n+1)-1)*Psi^(-n)*Q*M*(1-theta);
        fout(3,2) = n*xi^(1/(n+1))*Psi^(-n-1)*Q*M*(1-theta);
        fout(3,3) = -xi^(1/(n+1))*Psi^(-n)*M*(1-theta);
        
    case 2
        
        fout = zeros(3,3,3);
        
        fout(1,1,1) = 2*(n+1)^2/n^2*Q^(n+1);
        fout(1,1,3) = 2*(n+1)^3/n^2*xi*Q^n;
        fout(1,3,1) = fout(1,1,3);
        fout(1,3,3) = (n+1)^3/n*xi^2*Q^(n-1);
        
        fout(2,1,1) = ...
            + ((n^2*(m+1)-n)/(n+1)^2)*((n^2*(m+1)-n)/(n+1)^2-1)*C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2-2)*Psi^(-n)/4 ...
            + 1/(n+1)*(1/(n+1)-1)*2*M*xi^(1/(n+1)-2)*Psi^(1-n)*(1-theta) ...
            + n/(n+1)*(-n/(n+1)-1)*(1-r)*M*xi^(-n/(n+1)-2)*Psi^(-n)*(1-theta)/4;
        fout(2,1,2) = - (2*n+1-n^2)/n^2*Q^(n+1) ...
            - n*((n^2*(m+1)-n)/(n+1)^2)*C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2-1)*Psi^(-n-1)/4 ...
            + 1/(n+1)*(1-n)*2*M*xi^(1/(n+1)-1)*Psi^(-n)*(1-theta) ...
            - n^2/(n+1)*(1-r)*M*xi^(-n/(n+1)-1)*Psi^(-n-1)*(1-theta)/4;
        fout(2,2,1) = fout(2,1,2);
        fout(2,1,3) = - (n+1)*(2*n+1-n^2)/n^2*Q^n*Psi ...
            - ((n^2*(m+1)-n)/(n+1)^2)*C*theta*Q^(-2)*xi^((n^2*(m+1)-n)/(n+1)^2-1)*Psi^(-n)/4;
        fout(2,3,1) = fout(2,1,3);
        fout(2,2,2) =  + n*(n+1)*Psi^(-n-2)/4 ...
            + n*(n+1)*C*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n-2)/4 ...
            - n*(1-n)*2*M*xi^(1/(n+1))*Psi^(-n-1)*(1-theta) ...
            - n*(n+1)*(1-r)*M*xi^(-n/(n+1))*Psi^(-n-2)*(1-theta)/4;
        fout(2,2,3) =  - (n+1)*(2*n+1-n^2)/n^2*Q^n*xi ...
            + n*C*theta*Q^(-2)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n-1)/4;
        fout(2,3,2) = fout(2,2,3);
        fout(2,3,3) = - (n+1)*n*Q^(n-1)*(1-(1-theta)*r)/4 - (n+1)*n*(2*n+1-n^2)/n^2*Q^(n-1)*xi*Psi ...
            + 2*C*theta*Q^(-3)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n)/4;
        
        fout(3,1,1) = -1/(n+1)*(1/(n+1)-1)*xi^(1/(n+1)-2)*Psi^(-n)*Q*M*(1-theta);
        fout(3,1,2) = n/(n+1)*xi^(1/(n+1)-1)*Psi^(-n-1)*Q*M*(1-theta);
        fout(3,2,1) = fout(3,1,2);
        fout(3,1,3) = -1/(n+1)*xi^(1/(n+1)-1)*Psi^(-n)*M*(1-theta);
        fout(3,3,1) = fout(3,1,3);
        fout(3,2,2) = -n*(n+1)*xi^(1/(n+1))*Psi^(-n-2)*Q*M*(1-theta);
        fout(3,2,3) =  n*xi^(1/(n+1))*Psi^(-n-1)*M*(1-theta);
        fout(3,3,2) = fout(3,2,3);
       
    case 10
        
        type = parameters.type;
        fout = zeros(3,length(type));
        for ii = 1:length(type)
            switch type(ii)
                case 1
                    fout(2,ii) = theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n)/4;
                case 2
                    fout(2,ii) = 2*xi^(1/(n+1))*Psi^(1-n)*(1-theta) - (1-r)*xi^(-n/(n+1))*Psi^(-n)*(1-theta)/4;
                    fout(3,ii) = -xi^(1/(n+1))*Psi^(-n)*Q*(1-theta);
            end
        end
        
    case 11
        
        type = parameters.type;        
        fout = zeros(3,3,length(type));
        for ii = 1:length(type)
            switch type(ii)
                case 1
                    fout(2,1,ii) = ((n^2*(m+1)-n)/(n+1)^2)*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2-1)*Psi^(-n)/4;
                    fout(2,2,ii) = - n*theta*Q^(-1)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n-1)/4;
                    fout(2,3,ii) = - theta*Q^(-2)*xi^((n^2*(m+1)-n)/(n+1)^2)*Psi^(-n)/4;
                case 2
                    fout(2,1,ii) = 1/(n+1)*2*xi^(1/(n+1)-1)*Psi^(1-n)*(1-theta) ...
                        + n/(n+1)*(1-r)*xi^(-n/(n+1)-1)*Psi^(-n)*(1-theta)/4;
                    fout(2,2,ii) = (1-n)*2*xi^(1/(n+1))*Psi^(-n)*(1-theta) ...
                        + n*(1-r)*xi^(-n/(n+1))*Psi^(-n-1)*(1-theta)/4;
                    fout(3,1,ii) = -1/(n+1)*xi^(1/(n+1)-1)*Psi^(-n)*Q*(1-theta);
                    fout(3,2,ii) = n*xi^(1/(n+1))*Psi^(-n-1)*Q*(1-theta);
                    fout(3,3,ii) = -xi^(1/(n+1))*Psi^(-n)*(1-theta);
            end
        end        
end

end

